/* mbed Microcontroller Library 

 * Copyright (c) 2019 ARM Limited 

 * SPDX-License-Identifier: Apache-2.0 

 */ 

 
 



#include "mbed.h" 


// intialisation des pins correspondant aux 4 bobines du moteur pas à pas

DigitalOut pin_bobine_1_1(D2);   
DigitalOut pin_bobine_1_2(D3);
DigitalOut pin_bobine_2_1(D4);
DigitalOut pin_bobine_2_2(D5);


//pins permettant le fonctionnement des ponts
DigitalOut enableA(D7);
DigitalOut enableB(D8);

//initialisation du servomoteur 1
PwmOut servo_mot1(D9);


//initialisation du servomoteur 2
PwmOut servo_mot2(D10);

//Initialisation de la caméra de détection de présence
InterruptIn detector(D13);


//Initialisation comunication série
UnbufferedSerial      my_pc(USBTX, USBRX);
char color;
void ISR_my_pc_reception(void);
int col;
int a;


//Initialisation de la fonction d'interruption liée à la détection
void piston(void);  
  

void reglage_pins(int pin1, int pin2, int pin3, int pin4)    //Fonction permettant de régler la configuration des bobines pour le fonctionnement du moteur pas à pas
{
    pin_bobine_1_1 = pin1;
    pin_bobine_1_2 = pin2;
    pin_bobine_2_1 = pin3;
    pin_bobine_2_2 = pin4;
}


void prochainStep(int numero)  //fonction permettant de faire tourner le moteur pas à pas
{
    if (numero == 0) {
        reglage_pins(1, 0, 1, 0);
    }
    if (numero == 1) {
        reglage_pins(0, 1, 1, 0);
    }
    if (numero == 2) {
        reglage_pins(0, 1, 0, 1);
    }
    if (numero == 3) {
        reglage_pins(1, 0, 0, 1);
    }
}


void piston(void){

    if(col == 1){                   //fonction d'interruption permettant d'actionner le servomoteur au bon moment pour pousser la pièce
        servo_mot1.period_ms(10);         //
    

        for (int i=0; i <= 2050; i++){   //une fois la detection du front montant faite, il faut permettre de retarder l'action du servomoteur en fonction de la distance entre la caméra et le servomoteur
            prochainStep(i % 4);   //action permettant de passer d'un pas à l'autre du moteur pas à pas (la présence du "3 -" ou non impose le sens de rotation du tapis)
            wait_us(2500);               //wait réglant la vitesse du tapis roulant
        }
        servo_mot1.pulsewidth_us(2300);   //actionner le servomoteur pour pousser l'objet roulant

        for (int i=0; i <= 500; i++){   //temps d'attente nécessaire pour laisser le temps au moteur de bouger 
            prochainStep(i % 4); 
            wait_us(2500);              
        }
        servo_mot1.pulsewidth_us(700);   // Angle positif, permettant d'actionner le servomoteur pour remettre le piston à sa position initiale
        }
    
    else if(col == 2){                   //fonction d'interruption permettant d'actionner le servomoteur au bon moment pour pousser la pièce
        servo_mot2.period_ms(10);         //

        for (int i=0; i <= 900; i++){   //une fois la detection du front montant faite, il faut permettre de retarder l'action du servomoteur en fonction de la distance entre la caméra et le servomoteur
            prochainStep(i % 4);   //action permettant de passer d'un pas à l'autre du moteur pas à pas (la présence du "3 -" ou non impose le sens de rotation du tapis)
            wait_us(2500);               //wait réglant la vitesse du tapis roulant
        }
        servo_mot2.pulsewidth_us(600);   //actionner le servomoteur pour pousser l'objet roulant

        for (int i=0; i <= 600; i++){   //temps d'attente nécessaire pour laisser le temps au moteur de bouger 
            prochainStep(i % 4); 
            wait_us(2500);              
        }
        servo_mot2.pulsewidth_us(2200);   // Angle positif, permettant d'actionner le servomoteur pour remettre le piston à sa position initiale
    }

    else{
        a=1;  //effectuer une action quand même pour sortir de la boucle d'interruption quand on ne detecte rien ou qu'on detecte un bleu
    }

}





void ISR_my_pc_reception(void){
    my_pc.read(&color, 1);     // get the received byte
    if(color == 'R'){    col = 2;   } 
    if(color == 'G'){    col = 1;   }      
    my_pc.write(&color, 1);    // echo of the byte received
}







int main()
{   
    enableA = 1;              //validation du fonctionnement du pont A
    enableB = 1;              //validation du fonctionnement du pont B
    detector.rise(&piston);   //association de la fonction d'interruption à la detection d'un front montant
    my_pc.baud(115200);
    my_pc.attach(&ISR_my_pc_reception, UnbufferedSerial::RxIrq);
    int i=0;
    while (true){
        prochainStep(i % 4); 
        i++;
        wait_us(2500);
    }
}








 
 